@@ -4,7 +4,7 @@
  *
  * Implementation of DiscreteDistribution probability distribution.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -126,7 +126,13 @@ std::string DiscreteDistribution::ToString() const
 {
   std::ostringstream convert;
   convert << "DiscreteDistribution [" << this << "]" << std::endl;
-  convert << "Probabilities" << std::endl << probabilities;
+
+  // Secondary object so we can indent the probabilities.
+  std::ostringstream prob;
+  prob << "Probabilities:" << std::endl;
+  prob << probabilities;
+
+  convert << util::Indent(prob.str());
   return convert.str();
 }
 
