@@ -4,7 +4,7 @@
  *
  * Implementation of AugLagrangianTestFunction class.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -400,3 +400,11 @@ const arma::mat& LovaszThetaSDP::GetInitialPoint()
 
   return initialPoint;
 }
+
+std::string AugLagrangianTestFunction::ToString() const
+{
+  // The actual output is unimportant -- this object is only for testing anyway.
+  std::ostringstream convert;
+  convert << "AugLagrangianTestFunction [" << this << "]" << std::endl;
+  return convert.str();
+}
