@@ -4,7 +4,7 @@
  *
  * Definition of the Log class.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -19,8 +19,8 @@
  * You should have received a copy of the GNU General Public License along with
  * MLPACK.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifndef __MLPACK_CORE_IO_LOG_HPP
-#define __MLPACK_CORE_IO_LOG_HPP
+#ifndef __MLPACK_CORE_UTIL_LOG_HPP
+#define __MLPACK_CORE_UTIL_LOG_HPP
 
 #include <string>
 
