@@ -4,7 +4,7 @@
  *
  * Restricts a covariance matrix to being positive definite.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -22,6 +22,8 @@
 #ifndef __MLPACK_METHODS_GMM_POSITIVE_DEFINITE_CONSTRAINT_HPP
 #define __MLPACK_METHODS_GMM_POSITIVE_DEFINITE_CONSTRAINT_HPP
 
+#include <mlpack/core.hpp>
+
 namespace mlpack {
 namespace gmm {
 
@@ -39,13 +41,13 @@ class PositiveDefiniteConstraint
   static void ApplyConstraint(arma::mat& covariance)
   {
     // TODO: make this more efficient.
-    if (det(covariance) <= 1e-50)
+    if (arma::det(covariance) <= 1e-50)
     {
       Log::Debug << "Covariance matrix is not positive definite.  Adding "
           << "perturbation." << std::endl;
 
       double perturbation = 1e-30;
-      while (det(covariance) <= 1e-50)
+      while (arma::det(covariance) <= 1e-50)
       {
         covariance.diag() += perturbation;
         perturbation *= 10;
@@ -58,3 +60,4 @@ class PositiveDefiniteConstraint
 }; // namespace mlpack
 
 #endif
+
