@@ -4,7 +4,7 @@
  *
  * Simple least-squares linear regression.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -102,6 +102,9 @@ class LinearRegression
   //! Modify the Tikhonov regularization parameter for ridge regression.
   double& Lambda() { return lambda; }
 
+  // Returns a string representation of this object. 
+  std::string ToString() const;
+
  private:
   /**
    * The calculated B.
