@@ -4,7 +4,7 @@
  *
  * Implementation of Local Coordinate Coding
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -322,6 +322,16 @@ double LocalCoordinateCoding<DictionaryInitializer>::Objective(
   double froNormResidual = norm(data - dictionary * codes, "fro");
   return std::pow(froNormResidual, 2.0) + lambda * weightedL1NormZ;
 }
+template<typename DictionaryInitializer>
+std::string LocalCoordinateCoding<DictionaryInitializer>::ToString() const
+{
+  std::ostringstream convert;
+  convert << "Local Coordinate Coding [" << this << "]" << std::endl;
+  convert << "  Number of Atoms: " << atoms << std::endl;
+  convert << "  Data: " << data.n_rows << "x" << data.n_cols << std::endl;
+  convert << "  Lambda: " << lambda << std::endl;
+  return convert.str();
+}
 
 }; // namespace lcc
 }; // namespace mlpack
