@@ -4,7 +4,7 @@
  *
  * Main executable to run NMF.
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -21,15 +21,16 @@
  */
 #include <mlpack/core.hpp>
 
-#include "nmf.hpp"
+#include <mlpack/methods/amf/amf.hpp>
 
-#include "random_init.hpp"
-#include "mult_dist_update_rules.hpp"
-#include "mult_div_update_rules.hpp"
-#include "als_update_rules.hpp"
+#include <mlpack/methods/amf/init_rules/random_init.hpp>
+#include <mlpack/methods/amf/update_rules/nmf_mult_dist.hpp>
+#include <mlpack/methods/amf/update_rules/nmf_mult_div.hpp>
+#include <mlpack/methods/amf/update_rules/nmf_als.hpp>
+#include <mlpack/methods/amf/termination_policies/simple_residue_termination.hpp>
 
 using namespace mlpack;
-using namespace mlpack::nmf;
+using namespace mlpack::amf;
 using namespace std;
 
 // Document program.
@@ -119,26 +120,30 @@ int main(int argc, char** argv)
   {
     Log::Info << "Performing NMF with multiplicative distance-based update "
         << "rules." << std::endl;
-    NMF<> nmf(maxIterations, minResidue);
-    nmf.Apply(V, r, W, H);
+
+    SimpleResidueTermination srt(minResidue, maxIterations);
+    AMF<> amf(srt);
+    amf.Apply(V, r, W, H);
   }
   else if (updateRules == "multdiv")
   {
     Log::Info << "Performing NMF with multiplicative divergence-based update "
         << "rules." << std::endl;
-    NMF<RandomInitialization,
-        WMultiplicativeDivergenceRule,
-        HMultiplicativeDivergenceRule> nmf(maxIterations, minResidue);
-    nmf.Apply(V, r, W, H);
+    SimpleResidueTermination srt(minResidue, maxIterations);
+    AMF<SimpleResidueTermination, 
+        RandomInitialization, 
+        NMFMultiplicativeDivergenceUpdate> amf(srt);
+    amf.Apply(V, r, W, H);
   }
   else if (updateRules == "als")
   {
     Log::Info << "Performing NMF with alternating least squared update rules."
         << std::endl;
-    NMF<RandomInitialization,
-        WAlternatingLeastSquaresRule,
-        HAlternatingLeastSquaresRule> nmf(maxIterations, minResidue);
-    nmf.Apply(V, r, W, H);
+    SimpleResidueTermination srt(minResidue, maxIterations);
+    AMF<SimpleResidueTermination,
+        RandomInitialization, 
+        NMFALSUpdate> amf(srt);
+    amf.Apply(V, r, W, H);
   }
 
   // Save results.
