@@ -4,7 +4,7 @@
  *
  * Test file for SGD (stochastic gradient descent).
  *
- * This file is part of MLPACK 1.0.8.
+ * This file is part of MLPACK 1.0.10.
  *
  * MLPACK is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -24,6 +24,10 @@
 #include <mlpack/core/optimizers/lbfgs/test_functions.hpp>
 #include <mlpack/core/optimizers/sgd/test_function.hpp>
 
+#include <mlpack/core/metrics/ip_metric.hpp>
+#include <mlpack/core/metrics/lmetric.hpp>
+#include <mlpack/core/metrics/mahalanobis_distance.hpp>
+
 #include <boost/test/unit_test.hpp>
 #include "old_boost_test_definitions.hpp"
 
@@ -32,6 +36,7 @@ using namespace arma;
 using namespace mlpack;
 using namespace mlpack::optimization;
 using namespace mlpack::optimization::test;
+using namespace mlpack::metric;
 
 BOOST_AUTO_TEST_SUITE(SGDTest);
 
